import stripe


TEST_RESOURCE_ID = "bpc_123"


class TestConfiguration(object):
    def test_is_creatable(self, request_mock):
        resource = stripe.billing_portal.Configuration.create(
            business_profile={
                "privacy_policy_url": "https://example.com/privacy",
                "terms_of_service_url": "https://example.com/tos",
            },
            features={
                "customer_update": {
                    "allowed_updates": ["address"],
                    "enabled": True,
                }
            },
        )
        request_mock.assert_requested(
            "post", "/v1/billing_portal/configurations"
        )
        assert isinstance(resource, stripe.billing_portal.Configuration)

    def test_is_retrievable(self, request_mock):
        resource = stripe.billing_portal.Configuration.retrieve(
            TEST_RESOURCE_ID
        )
        request_mock.assert_requested(
            "get", "/v1/billing_portal/configurations/%s" % (TEST_RESOURCE_ID)
        )
        assert isinstance(resource, stripe.billing_portal.Configuration)

    def test_is_modifiable(self, request_mock):
        resource = stripe.billing_portal.Configuration.modify(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "post", "/v1/billing_portal/configurations/%s" % (TEST_RESOURCE_ID)
        )
        assert isinstance(resource, stripe.billing_portal.Configuration)

    def test_is_listable(self, request_mock):
        resource = stripe.billing_portal.Configuration.list()
        request_mock.assert_requested(
            "get", "/v1/billing_portal/configurations"
        )
        assert isinstance(
            resource.data[0], stripe.billing_portal.Configuration
        )



import stripe


TEST_RESOURCE_ID = "250FF"


class TestCoupon(object):
    def test_is_listable(self, request_mock):
        resources = stripe.Coupon.list()
        request_mock.assert_requested("get", "/v1/coupons")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.Coupon)

    def test_is_retrievable(self, request_mock):
        resource = stripe.Coupon.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v1/coupons/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Coupon)

    def test_is_creatable(self, request_mock):
        resource = stripe.Coupon.create(
            percent_off=25,
            duration="repeating",
            duration_in_months=3,
            id="250FF",
        )
        request_mock.assert_requested("post", "/v1/coupons")
        assert isinstance(resource, stripe.Coupon)

    def test_is_saveable(self, request_mock):
        resource = stripe.Coupon.retrieve(TEST_RESOURCE_ID)
        resource.metadata["key"] = "value"
        resource.save()
        request_mock.assert_requested(
            "post", "/v1/coupons/%s" % TEST_RESOURCE_ID
        )

    def test_is_modifiable(self, request_mock):
        resource = stripe.Coupon.modify(
            TEST_RESOURCE_ID, metadata={"key": "value"}
        )
        request_mock.assert_requested(
            "post", "/v1/coupons/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Coupon)

    def test_is_deletable(self, request_mock):
        resource = stripe.Coupon.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            "delete", "/v1/coupons/%s" % TEST_RESOURCE_ID
        )
        assert resource.deleted is True

    def test_can_delete(self, request_mock):
        resource = stripe.Coupon.delete(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "delete", "/v1/coupons/%s" % TEST_RESOURCE_ID
        )
        assert resource.deleted is True



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
# flake8: noqa

from stripe.api_resources.climate.order import Order as Order
from stripe.api_resources.climate.product import Product as Product
from stripe.api_resources.climate.supplier import Supplier as Supplier



import stripe


TEST_INVOICE_ID = "in_123"


class TestInvoiceLineItem(object):
    def test_deserialize(self, request_mock):
        invoice = stripe.Invoice.retrieve(TEST_INVOICE_ID)
        assert isinstance(invoice.lines.data, list)
        assert isinstance(invoice.lines.data[0], stripe.InvoiceLineItem)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
# flake8: noqa

from stripe.api_resources.issuing.authorization import (
    Authorization as Authorization,
)
from stripe.api_resources.issuing.card import Card as Card
from stripe.api_resources.issuing.cardholder import Cardholder as Cardholder
from stripe.api_resources.issuing.dispute import Dispute as Dispute
from stripe.api_resources.issuing.token import Token as Token
from stripe.api_resources.issuing.transaction import Transaction as Transaction



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
# flake8: noqa

from stripe.api_resources.billing_portal.configuration import (
    Configuration as Configuration,
)
from stripe.api_resources.billing_portal.session import Session as Session



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.api_resources.abstract import (
    CreateableAPIResource,
    ListableAPIResource,
)
from stripe.api_resources.expandable_field import ExpandableField
from stripe.api_resources.list_object import ListObject
from stripe.request_options import RequestOptions
from stripe.stripe_object import StripeObject
from typing import ClassVar, Dict, List, Optional, cast
from typing_extensions import Literal, NotRequired, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.api_resources.treasury.transaction import Transaction


class DebitReversal(
    CreateableAPIResource["DebitReversal"],
    ListableAPIResource["DebitReversal"],
):
    """
    You can reverse some [ReceivedDebits](https://stripe.com/docs/api#received_debits) depending on their network and source flow. Reversing a ReceivedDebit leads to the creation of a new object known as a DebitReversal.
    """

    OBJECT_NAME: ClassVar[
        Literal["treasury.debit_reversal"]
    ] = "treasury.debit_reversal"

    class LinkedFlows(StripeObject):
        issuing_dispute: Optional[str]
        """
        Set if there is an Issuing dispute associated with the DebitReversal.
        """

    class StatusTransitions(StripeObject):
        completed_at: Optional[int]
        """
        Timestamp describing when the DebitReversal changed status to `completed`.
        """

    class CreateParams(RequestOptions):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired["Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        received_debit: str
        """
        The ReceivedDebit to reverse.
        """

    class ListParams(RequestOptions):
        ending_before: NotRequired["str"]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        financial_account: str
        """
        Returns objects associated with this FinancialAccount.
        """
        limit: NotRequired["int"]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        received_debit: NotRequired["str"]
        """
        Only return DebitReversals for the ReceivedDebit ID.
        """
        resolution: NotRequired["Literal['lost', 'won']"]
        """
        Only return DebitReversals for a given resolution.
        """
        starting_after: NotRequired["str"]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """
        status: NotRequired["Literal['canceled', 'completed', 'processing']"]
        """
        Only return DebitReversals for a given status.
        """

    class RetrieveParams(RequestOptions):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """

    amount: int
    """
    Amount (in cents) transferred.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    financial_account: Optional[str]
    """
    The FinancialAccount to reverse funds from.
    """
    hosted_regulatory_receipt_url: Optional[str]
    """
    A [hosted transaction receipt](https://stripe.com/docs/treasury/moving-money/regulatory-receipts) URL that is provided when money movement is considered regulated under Stripe's money transmission licenses.
    """
    id: str
    """
    Unique identifier for the object.
    """
    linked_flows: Optional[LinkedFlows]
    """
    Other flows linked to a DebitReversal.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Dict[str, str]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    network: Literal["ach", "card"]
    """
    The rails used to reverse the funds.
    """
    object: Literal["treasury.debit_reversal"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    received_debit: str
    """
    The ReceivedDebit being reversed.
    """
    status: Literal["failed", "processing", "succeeded"]
    """
    Status of the DebitReversal
    """
    status_transitions: StatusTransitions
    transaction: Optional[ExpandableField["Transaction"]]
    """
    The Transaction associated with this object.
    """

    @classmethod
    def create(
        cls,
        api_key: Optional[str] = None,
        idempotency_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack[
            "DebitReversal.CreateParams"
        ]  # pyright: ignore[reportGeneralTypeIssues]
    ) -> "DebitReversal":
        """
        Reverses a ReceivedDebit and creates a DebitReversal object.
        """
        return cast(
            "DebitReversal",
            cls._static_request(
                "post",
                cls.class_url(),
                api_key,
                idempotency_key,
                stripe_version,
                stripe_account,
                params,
            ),
        )

    @classmethod
    def list(
        cls,
        api_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        **params: Unpack[
            "DebitReversal.ListParams"
        ]  # pyright: ignore[reportGeneralTypeIssues]
    ) -> ListObject["DebitReversal"]:
        """
        Returns a list of DebitReversals.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
            params=params,
        )
        if not isinstance(result, ListObject):

            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["DebitReversal.RetrieveParams"]
    ) -> "DebitReversal":
        """
        Retrieves a DebitReversal object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    _inner_class_types = {
        "linked_flows": LinkedFlows,
        "status_transitions": StatusTransitions,
    }



from stripe.api_resources.abstract.api_resource import APIResource
from typing import TypeVar, cast
from stripe.stripe_object import StripeObject

T = TypeVar("T", bound=StripeObject)


class CreateableAPIResource(APIResource[T]):
    @classmethod
    def create(
        cls,
        api_key=None,
        idempotency_key=None,
        stripe_version=None,
        stripe_account=None,
        **params
    ) -> T:
        return cast(
            T,
            cls._static_request(
                "post",
                cls.class_url(),
                api_key,
                idempotency_key,
                stripe_version,
                stripe_account,
                params,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
# flake8: noqa

from stripe.api_resources.treasury.credit_reversal import (
    CreditReversal as CreditReversal,
)
from stripe.api_resources.treasury.debit_reversal import (
    DebitReversal as DebitReversal,
)
from stripe.api_resources.treasury.financial_account import (
    FinancialAccount as FinancialAccount,
)
from stripe.api_resources.treasury.financial_account_features import (
    FinancialAccountFeatures as FinancialAccountFeatures,
)
from stripe.api_resources.treasury.inbound_transfer import (
    InboundTransfer as InboundTransfer,
)
from stripe.api_resources.treasury.outbound_payment import (
    OutboundPayment as OutboundPayment,
)
from stripe.api_resources.treasury.outbound_transfer import (
    OutboundTransfer as OutboundTransfer,
)
from stripe.api_resources.treasury.received_credit import (
    ReceivedCredit as ReceivedCredit,
)
from stripe.api_resources.treasury.received_debit import (
    ReceivedDebit as ReceivedDebit,
)
from stripe.api_resources.treasury.transaction import (
    Transaction as Transaction,
)
from stripe.api_resources.treasury.transaction_entry import (
    TransactionEntry as TransactionEntry,
)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.api_resources.abstract import UpdateableAPIResource
from stripe.api_resources.application_fee import ApplicationFee
from stripe.api_resources.expandable_field import ExpandableField
from typing import ClassVar, Dict, Optional, cast
from typing_extensions import Literal, TYPE_CHECKING
from urllib.parse import quote_plus

if TYPE_CHECKING:
    from stripe.api_resources.balance_transaction import BalanceTransaction


class ApplicationFeeRefund(UpdateableAPIResource["ApplicationFeeRefund"]):
    """
    `Application Fee Refund` objects allow you to refund an application fee that
    has previously been created but not yet refunded. Funds will be refunded to
    the Stripe account from which the fee was originally collected.

    Related guide: [Refunding application fees](https://stripe.com/docs/connect/destination-charges#refunding-app-fee)
    """

    OBJECT_NAME: ClassVar[Literal["fee_refund"]] = "fee_refund"
    amount: int
    """
    Amount, in cents (or local equivalent).
    """
    balance_transaction: Optional[ExpandableField["BalanceTransaction"]]
    """
    Balance transaction that describes the impact on your account balance.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    fee: ExpandableField["ApplicationFee"]
    """
    ID of the application fee that was refunded.
    """
    id: str
    """
    Unique identifier for the object.
    """
    metadata: Optional[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["fee_refund"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """

    @classmethod
    def _build_instance_url(cls, fee, sid):
        base = ApplicationFee.class_url()
        cust_extn = quote_plus(fee)
        extn = quote_plus(sid)
        return "%s/%s/refunds/%s" % (base, cust_extn, extn)

    @classmethod
    def modify(cls, fee, sid, **params) -> "ApplicationFeeRefund":
        url = cls._build_instance_url(fee, sid)
        return cast(
            "ApplicationFeeRefund",
            cls._static_request("post", url, params=params),
        )

    def instance_url(self):
        return self._build_instance_url(self.fee, self.id)

    @classmethod
    def retrieve(cls, id, api_key=None, **params) -> "ApplicationFeeRefund":
        raise NotImplementedError(
            "Can't retrieve a refund without an application fee ID. "
            "Use application_fee.refunds.retrieve('refund_id') instead."
        )



import stripe


TEST_RESOURCE_ID = "tu_123"


class TestTopup(object):
    def test_is_listable(self, request_mock):
        resources = stripe.Topup.list()
        request_mock.assert_requested("get", "/v1/topups")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.Topup)

    def test_is_retrievable(self, request_mock):
        resource = stripe.Topup.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v1/topups/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Topup)

    def test_is_creatable(self, request_mock):
        resource = stripe.Topup.create(
            amount=100,
            currency="usd",
            source="src_123",
            description="description",
            statement_descriptor="statement descriptor",
        )
        request_mock.assert_requested("post", "/v1/topups")
        assert isinstance(resource, stripe.Topup)

    def test_is_saveable(self, request_mock):
        resource = stripe.Topup.retrieve(TEST_RESOURCE_ID)
        resource.metadata["key"] = "value"
        resource.save()
        request_mock.assert_requested(
            "post", "/v1/topups/%s" % TEST_RESOURCE_ID
        )

    def test_is_modifiable(self, request_mock):
        resource = stripe.Topup.modify(
            TEST_RESOURCE_ID, metadata={"key": "value"}
        )
        request_mock.assert_requested(
            "post", "/v1/topups/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Topup)

    def test_can_cancel(self, request_mock):
        resource = stripe.Topup.retrieve(TEST_RESOURCE_ID)
        resource = resource.cancel()
        request_mock.assert_requested(
            "post", "/v1/topups/%s/cancel" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Topup)

    def test_can_cancel_classmethod(self, request_mock):
        resource = stripe.Topup.cancel(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "post", "/v1/topups/%s/cancel" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Topup)



import stripe


TEST_RESOURCE_ID = "issfr_123"


class TestEarlyFraudWarning(object):
    def test_is_listable(self, request_mock):
        resources = stripe.radar.EarlyFraudWarning.list()
        request_mock.assert_requested("get", "/v1/radar/early_fraud_warnings")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.radar.EarlyFraudWarning)

    def test_is_retrievable(self, request_mock):
        resource = stripe.radar.EarlyFraudWarning.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v1/radar/early_fraud_warnings/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.radar.EarlyFraudWarning)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
# flake8: noqa

from stripe.api_resources.tax.calculation import Calculation as Calculation
from stripe.api_resources.tax.calculation_line_item import (
    CalculationLineItem as CalculationLineItem,
)
from stripe.api_resources.tax.registration import Registration as Registration
from stripe.api_resources.tax.settings import Settings as Settings
from stripe.api_resources.tax.transaction import Transaction as Transaction
from stripe.api_resources.tax.transaction_line_item import (
    TransactionLineItem as TransactionLineItem,
)



import tempfile

import pytest

import stripe


TEST_RESOURCE_ID = "file_123"


class TestFile(object):
    @pytest.fixture(scope="function")
    def setup_upload_api_base(self):
        stripe.upload_api_base = stripe.api_base
        stripe.api_base = None
        yield
        stripe.api_base = stripe.upload_api_base
        stripe.upload_api_base = "https://files.stripe.com"

    def test_is_listable(self, request_mock):
        resources = stripe.File.list()
        request_mock.assert_requested("get", "/v1/files")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.File)

    def test_is_retrievable(self, request_mock):
        resource = stripe.File.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested("get", "/v1/files/%s" % TEST_RESOURCE_ID)
        assert isinstance(resource, stripe.File)

    def test_is_creatable(self, mocker):
        hc = mocker.Mock(stripe.http_client.HTTPClient)
        hc.name = "mockclient"
        stripe.default_http_client = hc
        hc.request_with_retries.return_value = ('{"object": "file"}', 200, {})
        stripe.multipart_data_generator.MultipartDataGenerator._initialize_boundary = (
            lambda self: 1234567890
        )
        test_file = tempfile.TemporaryFile()
        resource = stripe.File.create(
            purpose="dispute_evidence",
            file=test_file,
            file_link_data={"create": True},
        )
        method, url, headers, body = hc.request_with_retries.call_args[0]
        assert method == "post"
        assert url.endswith("/v1/files")
        assert (
            headers["Content-Type"]
            == "multipart/form-data; boundary=1234567890"
        )
        parts = body.split(b"--1234567890")
        assert len(parts) == 5
        assert parts[0] == b""
        assert b'name="purpose"' in parts[1]
        assert b'name="file"' in parts[2]
        assert b'name="file_link_data[create]"' in parts[3]
        assert isinstance(resource, stripe.File)

    def test_create_respects_stripe_version(
        self, setup_upload_api_base, request_mock
    ):
        test_file = tempfile.TemporaryFile()
        stripe.File.create(
            purpose="dispute_evidence", file=test_file, stripe_version="foo"
        )
        request_mock.assert_api_version("foo")

    # You can use api_version instead of stripe_version
    # in File.create. We preserve it for backwards compatibility
    def test_create_respects_api_version(
        self, setup_upload_api_base, request_mock
    ):
        test_file = tempfile.TemporaryFile()
        stripe.File.create(
            purpose="dispute_evidence", file=test_file, api_version="foo"
        )
        request_mock.assert_api_version("foo")

    def test_deserializes_from_file(self):
        obj = stripe.util.convert_to_stripe_object({"object": "file"})
        assert isinstance(obj, stripe.File)

    def test_deserializes_from_file_upload(self):
        obj = stripe.util.convert_to_stripe_object({"object": "file_upload"})
        assert isinstance(obj, stripe.File)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.stripe_object import StripeObject
from typing import ClassVar
from typing_extensions import Literal


class PlatformTaxFee(StripeObject):
    OBJECT_NAME: ClassVar[Literal["platform_tax_fee"]] = "platform_tax_fee"
    account: str
    """
    The Connected account that incurred this charge.
    """
    id: str
    """
    Unique identifier for the object.
    """
    object: Literal["platform_tax_fee"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    source_transaction: str
    """
    The payment object that caused this tax to be inflicted.
    """
    type: str
    """
    The type of tax (VAT).
    """



import stripe


TEST_RESOURCE_ID = "frr_123"


class TestReportRun(object):
    def test_is_creatable(self, request_mock):
        resource = stripe.reporting.ReportRun.create(
            parameters={"connected_account": "acct_123"},
            report_type="activity.summary.1",
        )
        request_mock.assert_requested("post", "/v1/reporting/report_runs")
        assert isinstance(resource, stripe.reporting.ReportRun)

    def test_is_listable(self, request_mock):
        resources = stripe.reporting.ReportRun.list()
        request_mock.assert_requested("get", "/v1/reporting/report_runs")
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.reporting.ReportRun)

    def test_is_retrievable(self, request_mock):
        resource = stripe.reporting.ReportRun.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            "get", "/v1/reporting/report_runs/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.reporting.ReportRun)
